%bcond_with blis
%bcond_without openblas

# https://bugzilla.redhat.com/show_bug.cgi?id=2058840
%undefine _ld_as_needed

%if %{with openblas}
%global default_backend openblas-openmp
%else
%global default_backend netlib
%endif
%global default_backend64 %{default_backend}64

%global major_version %%(echo %{version} | cut -d. -f1)
%global minor_version %%(echo %{version} | cut -d. -f2)
%global patch_version %%(echo %{version} | cut -d. -f3)

Summary:        A BLAS/LAPACK wrapper library with runtime exchangeable backends
Name:           flexiblas
Version:        3.3.1
Release:        3%{?dist}
License:        GPLv3 with exceptions and LGPLv2+ and BSD
URL:            https://www.mpi-magdeburg.mpg.de/projects/%{name}
Source0:        https://github.com/mpimd-csc/%{name}/archive/v%{version}/%{name}-%{version}.tar.gz

BuildRequires:  make cmake python gcc-fortran gcc-c++
BuildRequires:  blas-static lapack-static
BuildRequires:  atlas-devel
%if %{with blis}
BuildRequires:  blis-devel
%endif
%if %{with openblas}
BuildRequires:  openblas-devel
%endif

%description
FlexiBLAS is a wrapper library that enables the exchange of the BLAS and
LAPACK implementation used by a program without recompiling or relinking it.


%package        netlib
Summary:        FlexiBLAS wrapper library
Requires:       %{name} = %{version}-%{release}
Requires:       %{name}-%{default_backend} = %{version}-%{release}

%description    netlib
This package contains the wrapper library with 32-bit integer support for %{name}.


%package        hook-profile
Summary:        FlexiBLAS profile hook plugin
Requires:       %{name} = %{version}-%{release}
Requires:       %{name}-netlib = %{version}-%{release}

%description    hook-profile
This package contains a plugin that enables profiling support for %{name}.


%package        devel
Summary:        Development headers and libraries for FlexiBLAS
Requires:       %{name} = %{version}-%{release}

%description    devel
This package contains the development headers and libraries for %{name}.


%package        atlas
Summary:        FlexiBLAS wrappers for ATLAS
Requires:       %{name} = %{version}-%{release}

%description    atlas
This package contains FlexiBLAS wrappers for the ATLAS project.

%if %{with blis}
%package        blis-serial
Summary:        FlexiBLAS wrappers for BLIS
Requires:       %{name} = %{version}-%{release}

%description    blis-serial
This package contains FlexiBLAS wrappers for the sequential library compiled
with a 32-integer interface.


%package        blis-openmp
Summary:        FlexiBLAS wrappers for BLIS
Requires:       %{name} = %{version}-%{release}

%description    blis-openmp
This package contains FlexiBLAS wrappers for the library compiled with
OpenMP support with a 32-integer interface.


%package        blis-threads
Summary:        FlexiBLAS wrappers for BLIS
Requires:       %{name} = %{version}-%{release}

%description    blis-threads
This package contains FlexiBLAS wrappers for the library compiled with
threading support with a 32-integer interface.
%endif

%if %{with openblas}
%package        openblas-serial
Summary:        FlexiBLAS wrappers for OpenBLAS
Requires:       %{name} = %{version}-%{release}

%description    openblas-serial
This package contains FlexiBLAS wrappers for the sequential library compiled
with a 32-integer interface.


%package        openblas-openmp
Summary:        FlexiBLAS wrappers for OpenBLAS
Requires:       %{name} = %{version}-%{release}

%description    openblas-openmp
This package contains FlexiBLAS wrappers for the library compiled with
OpenMP support with a 32-integer interface.


%package        openblas-threads
Summary:        FlexiBLAS wrappers for OpenBLAS
Requires:       %{name} = %{version}-%{release}

%description    openblas-threads
This package contains FlexiBLAS wrappers for the library compiled with
threading support with a 32-integer interface.
%endif


%package        netlib64
Summary:        FlexiBLAS wrapper library (64-bit)
Requires:       %{name} = %{version}-%{release}
Requires:       %{name}-%{default_backend64} = %{version}-%{release}

%description    netlib64
This package contains the wrapper library with 64-bit integer support for %{name}.


%package        hook-profile64
Summary:        FlexiBLAS profile hook plugin (64-bit)
Requires:       %{name} = %{version}-%{release}
Requires:       %{name}-netlib64 = %{version}-%{release}

%description    hook-profile64
This package contains a plugin that enables profiling support for %{name}.


%if %{with blis}
%package        blis-serial64
Summary:        FlexiBLAS wrappers for BLIS (64-bit)
Requires:       %{name} = %{version}-%{release}

%description    blis-serial64
This package contains FlexiBLAS wrappers for the sequential library compiled
with a 64-integer interface.


%package        blis-openmp64
Summary:        FlexiBLAS wrappers for BLIS (64-bit)
Requires:       %{name} = %{version}-%{release}

%description    blis-openmp64
This package contains FlexiBLAS wrappers for the library compiled with
OpenMP support with a 64-integer interface.


%package        blis-threads64
Summary:        FlexiBLAS wrappers for BLIS (64-bit)
Requires:       %{name} = %{version}-%{release}

%description    blis-threads64
This package contains FlexiBLAS wrappers for the library compiled with
threading support with a 64-integer interface.
%endif


%if %{with openblas}
%package        openblas-serial64
Summary:        FlexiBLAS wrappers for OpenBLAS (64-bit)
Requires:       %{name} = %{version}-%{release}

%description    openblas-serial64
This package contains FlexiBLAS wrappers for the sequential library compiled
with a 64-integer interface.


%package        openblas-openmp64
Summary:        FlexiBLAS wrappers for OpenBLAS (64-bit)
Requires:       %{name} = %{version}-%{release}

%description    openblas-openmp64
This package contains FlexiBLAS wrappers for the library compiled with
OpenMP support with a 64-integer interface.


%package        openblas-threads64
Summary:        FlexiBLAS wrappers for OpenBLAS (64-bit)
Requires:       %{name} = %{version}-%{release}

%description    openblas-threads64
This package contains FlexiBLAS wrappers for the library compiled with
threading support with a 64-integer interface.
%endif


%prep
%autosetup


%build
rm -rf contributed
%cmake -B build \
    -DCMAKE_INSTALL_PREFIX=%{_prefix} \
    -DSYS_BLAS_LIBRARY=$(pkg-config --variable=libdir blas)/libblas.a \
    -DSYS_LAPACK_LIBRARY=$(pkg-config --variable=libdir lapack)/liblapack_pic.a \
    -DINTEGER8=OFF \
    -DCMAKE_SKIP_INSTALL_RPATH=ON \
    -DTESTS=ON

%make_build -C build
%cmake -B build64 \
    -DCMAKE_INSTALL_PREFIX=%{_prefix} \
    -DSYS_BLAS_LIBRARY=$(pkg-config --variable=libdir blas)/libblas64.a \
    -DSYS_LAPACK_LIBRARY=$(pkg-config --variable=libdir lapack)/liblapack_pic64.a \
    -DINTEGER8=ON \
    -DCMAKE_SKIP_INSTALL_RPATH=ON \
    -DTESTS=ON
%make_build -C build64


%install
%make_install -C build
echo "default = %{default_backend}" > %{buildroot}%{_sysconfdir}/%{name}rc
%make_install -C build64
echo "default = %{default_backend64}" > %{buildroot}%{_sysconfdir}/%{name}64rc

rm -f %{buildroot}%{_libdir}/%{name}*/lib%{name}_hook_dummy.so

rename -- serial -serial %{buildroot}%{_libdir}/%{name}*/* || true
rename -- openmp -openmp %{buildroot}%{_libdir}/%{name}*/* || true
rename -- pthread -threads %{buildroot}%{_libdir}/%{name}*/* || true
rename NETLIB netlib %{buildroot}%{_sysconfdir}/%{name}*.d/* || true
rename ATLAS atlas %{buildroot}%{_sysconfdir}/%{name}*.d/* || true
rename Blis blis %{buildroot}%{_sysconfdir}/%{name}*.d/* || true
rename OpenBLAS openblas %{buildroot}%{_sysconfdir}/%{name}*.d/* || true
rename -- Serial -serial %{buildroot}%{_sysconfdir}/%{name}*.d/* || true
rename -- OpenMP -openmp %{buildroot}%{_sysconfdir}/%{name}*.d/* || true
rename -- PThread -threads %{buildroot}%{_sysconfdir}/%{name}*.d/* || true
find %{buildroot}%{_sysconfdir}/%{name}*.d/* -type f \
    -exec sed -i 's NETLIB netlib gI' {} \;\
    -exec sed -i 's ATLAS atlas gI' {} \;\
    -exec sed -i 's Blis blis gI' {} \;\
    -exec sed -i 's OpenBLAS openblas gI' {} \;\
    -exec sed -i 's Serial -serial gI' {} \;\
    -exec sed -i 's OpenMP -openmp gI' {} \;\
    -exec sed -i 's PThread -threads gI' {} \;


%check
export FLEXIBLAS_TEST=%{buildroot}%{_libdir}/%{name}/lib%{name}_%{default_backend}.so
make -C build test
export FLEXIBLAS64_TEST=%{buildroot}%{_libdir}/%{name}64/lib%{name}_%{default_backend64}.so
make -C build64 test


%files
%license COPYING COPYING.NETLIB
%doc ISSUES.md README.md CHANGELOG


%files netlib
%config(noreplace) %{_sysconfdir}/%{name}rc
%dir %{_sysconfdir}/%{name}rc.d
%{_sysconfdir}/%{name}rc.d/netlib.conf
%{_bindir}/%{name}
%{_libdir}/lib%{name}.so.%{major_version}
%{_libdir}/lib%{name}.so.%{major_version}.%{minor_version}
%{_libdir}/lib%{name}_api.so.%{major_version}
%{_libdir}/lib%{name}_api.so.%{major_version}.%{minor_version}
%{_libdir}/lib%{name}_mgmt.so.%{major_version}
%{_libdir}/lib%{name}_mgmt.so.%{major_version}.%{minor_version}
%dir %{_libdir}/%{name}
%{_libdir}/%{name}/lib%{name}_fallback_lapack.so
%{_libdir}/%{name}/lib%{name}_netlib.so
%{_mandir}/man1/%{name}.1*


%files hook-profile
%{_libdir}/%{name}/lib%{name}_hook_profile.so


%files devel
%{_bindir}/%{name}-config
%{_includedir}/%{name}
%{_libdir}/lib%{name}.so
%{_libdir}/lib%{name}_api.so
%{_libdir}/lib%{name}_mgmt.so
%{_libdir}/pkgconfig/%{name}.pc
%{_libdir}/pkgconfig/%{name}_api.pc
%{_bindir}/%{name}64-config
%{_includedir}/%{name}64
%{_libdir}/lib%{name}64.so
%{_libdir}/lib%{name}64_api.so
%{_libdir}/lib%{name}64_mgmt.so
%{_libdir}/pkgconfig/%{name}64.pc
%{_libdir}/pkgconfig/%{name}64_api.pc
%{_mandir}/man3/%{name}_*
%{_mandir}/man7/%{name}-api.7*


%files atlas
%{_sysconfdir}/%{name}rc.d/atlas.conf
%{_libdir}/%{name}/lib%{name}_atlas.so


%if %{with blis}
%files blis-serial
%{_sysconfdir}/%{name}rc.d/blis-serial.conf
%{_libdir}/%{name}/lib%{name}_blis-serial.so

%files blis-openmp
%{_sysconfdir}/%{name}rc.d/blis-openmp.conf
%{_libdir}/%{name}/lib%{name}_blis-openmp.so

%files blis-threads
%{_sysconfdir}/%{name}rc.d/blis-threads.conf
%{_libdir}/%{name}/lib%{name}_blis-threads.so
%endif


%if %{with openblas}
%files openblas-serial
%{_sysconfdir}/%{name}rc.d/openblas-serial.conf
%{_libdir}/%{name}/lib%{name}_openblas-serial.so

%files openblas-openmp
%{_sysconfdir}/%{name}rc.d/openblas-openmp.conf
%{_libdir}/%{name}/lib%{name}_openblas-openmp.so

%files openblas-threads
%{_sysconfdir}/%{name}rc.d/openblas-threads.conf
%{_libdir}/%{name}/lib%{name}_openblas-threads.so
%endif


%files netlib64
%config(noreplace) %{_sysconfdir}/%{name}64rc
%dir %{_sysconfdir}/%{name}64rc.d
%{_sysconfdir}/%{name}64rc.d/netlib.conf
%{_bindir}/%{name}64
%{_libdir}/lib%{name}64.so.%{major_version}
%{_libdir}/lib%{name}64.so.%{major_version}.%{minor_version}
%{_libdir}/lib%{name}64_api.so.%{major_version}
%{_libdir}/lib%{name}64_api.so.%{major_version}.%{minor_version}
%{_libdir}/lib%{name}64_mgmt.so.%{major_version}
%{_libdir}/lib%{name}64_mgmt.so.%{major_version}.%{minor_version}
%dir %{_libdir}/%{name}64
%{_libdir}/%{name}64/lib%{name}_fallback_lapack.so
%{_libdir}/%{name}64/lib%{name}_netlib.so
%{_mandir}/man1/%{name}64.1*


%files hook-profile64
%{_libdir}/%{name}64/lib%{name}_hook_profile.so


%if %{with blis}
%files blis-serial64
%{_sysconfdir}/%{name}64rc.d/blis-serial64.conf
%{_libdir}/%{name}64/lib%{name}_blis-serial64.so

%files blis-openmp64
%{_sysconfdir}/%{name}64rc.d/blis-openmp64.conf
%{_libdir}/%{name}64/lib%{name}_blis-openmp64.so

%files blis-threads64
%{_sysconfdir}/%{name}64rc.d/blis-threads64.conf
%{_libdir}/%{name}64/lib%{name}_blis-threads64.so
%endif


%if %{with openblas}
%files openblas-serial64
%{_sysconfdir}/%{name}64rc.d/openblas-serial64.conf
%{_libdir}/%{name}64/lib%{name}_openblas-serial64.so

%files openblas-openmp64
%{_sysconfdir}/%{name}64rc.d/openblas-openmp64.conf
%{_libdir}/%{name}64/lib%{name}_openblas-openmp64.so

%files openblas-threads64
%{_sysconfdir}/%{name}64rc.d/openblas-threads64.conf
%{_libdir}/%{name}64/lib%{name}_openblas-threads64.so
%endif


%changelog
* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.3.1-3
- Rebuilt for OpenCloudOS Stream 23.09

* Fri Aug 4 2023 Shuo Wang <abushwang@tencent.com> - 3.3.1-2
- Undefine _ld_as_needed to fix underlinking issue for boost

* Wed Aug 2 2023 Shuo Wang <abushwang@tencent.com> - 3.3.1-1
- update to 3.3.1

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.1.3-5
- Rebuilt for OpenCloudOS Stream 23.05

* Mon Apr 24 2023 rockerzhu <rockerzhu@tencent.com> - 3.1.3-4
- Rebuild for openblas 0.3.23.

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.1.3-3
- Rebuilt for OpenCloudOS Stream 23

* Thu Feb 23 2023 Miaojun Dong <zoedong@tencent.com> - 3.1.3-2
- remove README.md

* Wed Nov 23 2022 rockerzhu <rockerzhu@tencent.com> - 3.1.3-1
- Initial build
